var Vector2D = (function () {
    function Vector2D(x, y) {
        var _this = this;
        this.clone = function () { return new Vector2D(_this.X, _this.Y); };
        /* Length */
        this.getLength = function () { return Math.sqrt((_this.X * _this.X) + (_this.Y * _this.Y)); };
        this.getLengthSquared = function () { return (_this.X * _this.X) + (_this.Y * _this.Y); };
        /* Normalization */
        this.normalize = function () {
            var mag = _this.getLength();
            _this.X = _this.X / mag;
            _this.Y = _this.Y / mag;
            return _this;
        };
        this.getNormalized = function () {
            var mag = _this.getLength();
            return new Vector2D(_this.X / mag, _this.Y / mag);
        };
        /* Addition */
        this.add = function (dx, dy) {
            _this.X += dx;
            _this.Y += dy;
            return _this;
        };
        this.addVec = function (v) {
            _this.X += v.X;
            _this.Y += v.Y;
            return _this;
        };
        this.getAdd = function (dx, dy) { return new Vector2D(_this.X + dx, _this.Y + dy); };
        this.getAddVec = function (v) { return new Vector2D(_this.X + v.X, _this.Y + v.Y); };
        /* Subtraction */
        this.sub = function (dx, dy) {
            _this.X -= dx;
            _this.Y -= dy;
            return _this;
        };
        this.subVec = function (v) {
            _this.X -= v.X;
            _this.Y -= v.Y;
            return _this;
        };
        this.getSub = function (dx, dy) { return new Vector2D(_this.X - dx, _this.Y - dy); };
        this.getSubVec = function (v) { return new Vector2D(_this.X - v.X, _this.Y - v.Y); };
        /* Multiplication */
        this.mul = function (s) { _this.X *= s; _this.Y *= s; return _this; };
        this.getMul = function (s) { return new Vector2D(_this.X * s, _this.Y * s); };
        /* Division */
        this.div = function (s) { _this.X /= s; _this.Y /= s; return _this; };
        this.getDiv = function (s) { return new Vector2D(_this.X / s, _this.Y / s); };
        /* Dot product */
        this.dotProduct = function (v) { return ((_this.X * v.X) + (_this.Y * v.Y)); };
        /* Perp dot product */
        this.perpDotProduct = function (v) { return ((_this.X * v.Y) - (_this.Y * v.X)); };
        /* Opposite */
        this.opposite = function () { _this.X = -_this.X; _this.Y = -_this.Y; return _this; };
        this.getOpposite = function () { return new Vector2D(-_this.X, -_this.Y); };
        /* Perpendicular */
        this.perpendicularCCW = function () { _this.X = -_this.Y; _this.Y = _this.X; return _this; };
        this.getPerpendicularCCW = function () { return new Vector2D(-_this.Y, _this.X); };
        this.perpendicularCW = function () { _this.X = _this.Y; _this.Y = -_this.X; return _this; };
        this.getPerpendicularCW = function () { return new Vector2D(_this.Y, -_this.X); };
        /* IsOrthogonal */
        this.IsOrthogonal = function (v) { return _this.dotProduct(v) == 0; };
        /* IsColinear */
        this.isColinear = function (v) { return _this.perpDotProduct(v) == 0; };
        /* Rotate */
        this.rotate = function (angle) {
            var sin = Math.sin(angle);
            var cos = Math.cos(angle);
            var newX = _this.X * cos - _this.Y * sin;
            var newY = _this.X * sin + _this.Y * cos;
            _this.X = newX;
            _this.Y = newY;
            return _this;
        };
        this.getRotated = function (angle) {
            var sin = Math.sin(angle);
            var cos = Math.cos(angle);
            var newX = _this.X * cos - _this.Y * sin;
            var newY = _this.X * sin + _this.Y * cos;
            return new Vector2D(newX, newY);
        };
        /* Polar */
        this.getVectorFromPolar = function (mag, angle) {
            return new Vector2D(mag * Math.cos(angle), mag * Math.sin(angle));
        };
        this.getPolarMagnitude = function () { return _this.getLength(); };
        this.getPolarAngle = function () {
            if (_this.X == 0) {
                if (_this.Y > 0) {
                    return Math.PI / 2;
                }
                else {
                    return -Math.PI / 2;
                }
            }
            else if (_this.X < 0) {
                if (_this.Y >= 0) {
                    return Math.atan(_this.Y / _this.X) + Math.PI / 2;
                }
                else {
                    return Math.atan(_this.Y / _this.X) - Math.PI / 2;
                }
            }
            else {
                return Math.atan(_this.Y / _this.X);
            }
        };
        /* Projected */
        this.getProjectLength = function (v) {
            return (_this.dotProduct(v) / _this.getLength());
        };
        this.getProjectVector = function (v) {
            return _this.getNormalized().getMul(_this.dotProduct(v) / _this.getLength());
        };
        if (typeof x == "number") {
            this.X = x;
            this.Y = y || 0;
        }
        else if (x instanceof Vector2D) {
            this.X = x.X;
            this.Y = x.Y;
        }
        else {
            this.X = 0;
            this.Y = 0;
        }
    }
    return Vector2D;
}());
//# sourceMappingURL=Vector2D.js.map