/******************************************************************************************************************************************
 *
 * Filename    : Program.cs
 * Project     : VectorReflection
 * Description : This is an visualization how a vector is reflected at a plane. This visualiztion is in 2D, but the used mathematics
 *               also applies for higher dimensions.
 *
 * Sunshine, March 2k21
 * www.sunshine2k.de | www.bastian-molkenthin.de
 *
 * This file is licensed under MIT licence. For further details, see http://www.sunshine2k.de/license.html.
 *
 * Update October 2k23: Fix issue with two calls of requestAnimationFrame()
 *****************************************************************************************************************************************/
var cvs;
var myApp;
var VectorReflectionApp = (function () {
    function VectorReflectionApp(cvs) {
        var _this = this;
        /**
         * Update loop that does cyclically all calculation and drawing.
         */
        this.update = function (time) {
            _this.calculate();
            _this.paint(time);
            window.requestAnimationFrame(_this.update);
        };
        /**
         * Performs the actual vector calculation
         */
        this.calculate = function () {
            /* calculate plane vector and plane normal (normalized!) */
            _this.vPlane = new Vector2D(_this.planePoint2.getX() - _this.planePoint1.getX(), _this.planePoint2.getY() - _this.planePoint1.getY());
            _this.vPlaneNormal = _this.vPlane.getPerpendicularCW().normalize();
            /* calculate plane midpoint */
            _this.vPlaneMidpoint = new Vector2D(_this.planePoint1.getX() + (_this.planePoint2.getX() - _this.planePoint1.getX()) * 0.5, _this.planePoint1.getY() + (_this.planePoint2.getY() - _this.planePoint1.getY()) * 0.5);
            /* calculate source vector (to be projected) from its points */
            _this.vSource = new Vector2D(_this.vPlaneMidpoint.X - _this.sourceVecPoint1.getX(), _this.vPlaneMidpoint.Y - _this.sourceVecPoint1.getY());
            /* calculate dot product of source vector and plane normal vector */
            var dotSourceNormal = _this.vSource.dotProduct(_this.vPlaneNormal);
            /* calculate actual reflected vector */
            _this.vReflection = _this.vSource.getSubVec(_this.vPlaneNormal.getMul(2.0 * dotSourceNormal));
        };
        /**
         * Performs all drawing
         */
        this.paint = function (time) {
            _this.cvsContext.save();
            /* fill background */
            _this.cvsContext.fillStyle = "#ffdda8";
            _this.cvsContext.fillRect(0, 0, _this.canvasHtmlElement.width, _this.canvasHtmlElement.height);
            /* draw plane line */
            _this.cvsContext.strokeStyle = "#646464";
            _this.drawLine(_this.planePoint1.getX(), _this.planePoint1.getY(), _this.planePoint2.getX(), _this.planePoint2.getY());
            /* draw plane normal in the middle of the plane */
            _this.cvsContext.strokeStyle = "#CCCCCC";
            _this.drawLine(_this.vPlaneMidpoint.X, _this.vPlaneMidpoint.Y, _this.vPlaneMidpoint.X + (_this.vPlaneNormal.X * 100), _this.vPlaneMidpoint.Y + (_this.vPlaneNormal.Y * 100));
            /* draw input vector */
            var sourceVector = new Vector2D(_this.vPlaneMidpoint.X - _this.sourceVecPoint1.getX(), _this.vPlaneMidpoint.Y - _this.sourceVecPoint1.getY());
            _this.cvsContext.strokeStyle = "#6de900";
            _this.drawLine(_this.vPlaneMidpoint.X, _this.vPlaneMidpoint.Y, _this.vPlaneMidpoint.X - sourceVector.X, _this.vPlaneMidpoint.Y - sourceVector.Y);
            /* draw reflected vector */
            var vReflectionNormal = _this.vReflection.getNormalized().mul(_this.vSource.getLength());
            _this.cvsContext.strokeStyle = "#03c6fc";
            _this.drawLine(_this.vPlaneMidpoint.X, _this.vPlaneMidpoint.Y, _this.vPlaneMidpoint.X + vReflectionNormal.X, _this.vPlaneMidpoint.Y + vReflectionNormal.Y);
            /* draw points */
            _this.cvsContext.strokeStyle = "#000000";
            _this.cvsContext.lineWidth = 0.7;
            for (var i = 0; i < _this.points.length; i++) {
                _this.points[i].draw();
            }
            _this.cvsContext.restore();
        };
        this.drawLine = function (x1, y1, x2, y2) {
            _this.cvsContext.beginPath();
            _this.cvsContext.lineWidth = 1;
            _this.cvsContext.moveTo(x1, y1);
            _this.cvsContext.lineTo(x2, y2);
            _this.cvsContext.stroke();
        };
        this.mouseDown = function (event) {
            var cursorPosX = event.x - _this.canvasHtmlElement.offsetLeft;
            var cursorPosY = event.y - _this.canvasHtmlElement.offsetTop;
            for (var i = 0; i < _this.points.length; i++) {
                if (_this.points[i].startDragging(cursorPosX, cursorPosY)) {
                    _this.canvasHtmlElement.style.cursor = "pointer";
                    break;
                }
            }
        };
        this.mouseUp = function (event) {
            for (var i = 0; i < _this.points.length; i++) {
                _this.points[i].stopDragging();
            }
            /* change cursor back to normal */
            _this.canvasHtmlElement.style.cursor = "default";
        };
        this.mouseMove = function (event) {
            var cursorPosX = event.x - _this.canvasHtmlElement.offsetLeft;
            var cursorPosY = event.y - _this.canvasHtmlElement.offsetTop;
            var pointHit = false;
            /* inform each point about new cursor position */
            for (var i = 0; i < _this.points.length; i++) {
                _this.points[i].cursorMoved(cursorPosX, cursorPosY);
                /* if cursor is over point, change cursor to hand */
                if (_this.points[i].isCursorOnPoint(cursorPosX, cursorPosY)) {
                    _this.canvasHtmlElement.style.cursor = "pointer";
                    pointHit = true;
                }
            }
            /* if cursor is over no point, use normal cursor */
            if (!pointHit) {
                _this.canvasHtmlElement.style.cursor = "default";
            }
        };
        this.canvasHtmlElement = cvs;
        this.cvsContext = this.canvasHtmlElement.getContext("2d");
        this.planePoint1 = new MovablePoint(this.cvsContext, 100, 300, "");
        this.planePoint2 = new MovablePoint(this.cvsContext, 400, 200, "");
        this.sourceVecPoint1 = new MovablePoint(this.cvsContext, 120, 180, "");
        this.points = new Array();
        this.points.push(this.planePoint1);
        this.points.push(this.planePoint2);
        this.points.push(this.sourceVecPoint1);
        this.canvasHtmlElement.addEventListener("mousedown", this.mouseDown, false);
        this.canvasHtmlElement.addEventListener("mouseup", this.mouseUp, false);
        this.canvasHtmlElement.addEventListener("mousemove", this.mouseMove, false);
        this.vReflection = new Vector2D(0, 0);
        this.update(0);
    }
    return VectorReflectionApp;
}());
window.addEventListener("load", function () {
    cvs = document.getElementById('cvs');
    myApp = new VectorReflectionApp(cvs);
});
//# sourceMappingURL=app.js.map