﻿class MovablePoint
{
    public static WIDTH_HALF: number = 5;

    private ctx: CanvasRenderingContext2D;
    private x: number;
    private y: number;
    private name: string;
    private isDragging: boolean;

    constructor(ctx: CanvasRenderingContext2D, x: number, y: number, name: string)
    {
        this.ctx = ctx;
        this.x = x;
        this.y = y;
        this.name = name;
        this.isDragging = false;
    }

    public getX(): number { return this.x; }
    public getY(): number { return this.y; }
    public setX(v: number): void { this.x = v; }
    public setY(v: number): void { this.y = v; }

    public draw(): void
    {
        this.ctx.fillStyle = "#555555";
        this.ctx.fillRect(this.x - MovablePoint.WIDTH_HALF, this.y - MovablePoint.WIDTH_HALF,
            2 * MovablePoint.WIDTH_HALF, 2 * MovablePoint.WIDTH_HALF);

        if (this.name)
        {
            this.ctx.save();
            this.ctx.font = '12px Roboto';
            this.ctx.strokeText(this.name + " (" + this.x + ", " + this.y + ")",
                this.x + 2 * MovablePoint.WIDTH_HALF, this.y - 2 * MovablePoint.WIDTH_HALF);
            this.ctx.restore();
        }
    }

    public isCursorOnPoint(cursorX: number, cursorY: number): boolean
    {
        if ((cursorX >= this.x - MovablePoint.WIDTH_HALF) && (cursorX <= this.x + MovablePoint.WIDTH_HALF) &&
            (cursorY >= this.y - MovablePoint.WIDTH_HALF) && (cursorY <= this.y + MovablePoint.WIDTH_HALF))
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    public startDragging(cursorX: number, cursorY: number): boolean
    {
        if ((cursorX >= this.x - MovablePoint.WIDTH_HALF) && (cursorX <= this.x + MovablePoint.WIDTH_HALF) &&
            (cursorY >= this.y - MovablePoint.WIDTH_HALF) && (cursorY <= this.y + MovablePoint.WIDTH_HALF))
        {
            this.isDragging = true;
            return true;
        }
        else
        {
            return false;
        }
    }

    public stopDragging(): void
    {
        this.isDragging = false;
    }

    public cursorMoved(cursorX: number, cursorY: number)
    {
        if (this.isDragging)
        {
            this.x = cursorX;
            this.y = cursorY;
        }
    }
}